;20170216 Ali
;line plots and 3D representations of model and data at the times specified by cursor
;keywords:
;avrg: for averaged values over a time period
;tplot: for selecting the data from the current tplot window

pro mvn_pui_plot_tsample,all=all,tplot=tplot,avrg=avrg,overplot=overplot,xrange=xr,yrange=yr,zeros=zeros,sep=sep,swe=swe,traj=traj,eflux=eflux,swia3d=swia3d,stah3d=stah3d,stao3d=stao3d

@mvn_pui_commonblock.pro ;common mvn_pui_common

if n_elements(zeros) eq 0 then zeros=1e-7 ;to fix the stairs

if keyword_set(tplot) then begin
  var=tsample(val=en,aver=avrg,/silent)
  var[where(var eq 0,/null)]=zeros
  p=plot(en,var,/xlog,/ylog,/stairs,xrange=xr,yrange=yr,xtickunits='scientific',ytickunits='scientific',overplot=overplot)
  return
endif

if keyword_set(avrg) then np=2 else np=1

if keyword_set(all) then begin
  sep=1
  swe=1
  traj=1
  eflux=1
endif

ctime,t,np=np,/silent
tstep=floor((t-pui[0].centertime)/pui0.tbin)
if n_elements(tstep) eq 1 then tstep=[tstep,tstep]
tsteps=lindgen(1l+tstep[1]-tstep[0],start=tstep[0])

rv=1e-3*transpose(average(pui[tsteps].model.rv,4,/nan),[1,2,0]) ;rv (km, km/s)
seprv=1e-3*transpose(average(pui[tsteps].model[1].fluxes.sep.rv,3,/nan)) ;sep rv (km, km/s)
seprv[where(~finite(seprv),/null)]=0. ;for scatterplot to handle NaN's
sepfov=average(pui[tsteps].data.sep.fov,3,/nan) ;sep fov
sep1fov=sepfov[*,0]
sep2fov=sepfov[*,1]
swiyfov=crossp(sep1fov,sep2fov)
vxyz=reform(rv[*,1,3:5])
vtot=sqrt(total(vxyz^2,2))
cosvsep1=-total(vxyz*(replicate(1.,pui0.np)#sep1fov),2)/vtot
cosvsep2=-total(vxyz*(replicate(1.,pui0.np)#sep2fov),2)/vtot
cosvswiy=-total(vxyz*(replicate(1.,pui0.np)#swiyfov),2)/vtot

if keyword_set(sep) then begin ;SEP
  sepet=pui1.sepet.sepbo ;sep energy table (keV)
  sepie=.5+findgen(500) ;sep incident energy table (keV)
  entot=pui1.totet/1e3 ;tot energy table (keV)

  sepd=average(pui[tsteps].data.sep.rate_bo,3,/nan) ;sep data
  sepm=average(pui[tsteps].model[1].fluxes.sep.model_rate,3,/nan) ;sep model
  sepi=average(pui[tsteps].model[1].fluxes.sep.incident_rate,3,/nan) ;incident in sep fov /[s keV]
  ftot=average(pui[tsteps].model[1].fluxes.toteflux,2,/nan)/entot ;total flux /[s cm2 keV]

  sepd[where(sepd eq 0.,/null)]=zeros
  sepm[where(sepm eq 0.,/null)]=zeros
  sepi[where(sepi eq 0.,/null)]=zeros
  ftot[where(ftot eq 0.,/null)]=zeros

  xr=[0,1e4]
  yr=[.01,1e4]
  xt='Energy (keV)'
  yt='Count Rate/Energy Bin (Hz)'
  ;SEP1F
  p=plot([0],/nodata,/xlog,/ylog,xrange=xr,yrange=yr,margin=.1,layout=[2,1,1],title='SEP1F',xtitle=xt,ytitle=yt,xtickunits='scientific',ytickunits='scientific')
  p=plot(/o,sepet[*,0],sepd[*,0],/stairs,'r')
  p=plot(/o,sepet[*,0],sepm[*,0],/stairs,'b')
  p=plot(/o,sepie,sepi[*,0],'m')
  p=plot(/o,entot,ftot,'g')
  ;SEP2F
  p=plot([0],/nodata,/xlog,/ylog,xrange=xr,yrange=yr,margin=.1,layout=[2,1,2],title='SEP2F',xtitle=xt,ytitle=yt,xtickunits='scientific',ytickunits='scientific',/current)
  p=plot(/o,sepet[*,1],sepd[*,1],/stairs,'r')
  p=plot(/o,sepet[*,1],sepm[*,1],/stairs,'b')
  p=plot(/o,sepie,sepi[*,1],'m')
  p=plot(/o,entot,ftot,'g')
  p=mvn_pui_sep_angular_response(cosvsep1,cosvsep2,cosvswiy,/trajplot,tsteps=tsteps)
endif

if keyword_set(swe) then begin ;SWEA/SWIA
  xswepot=average(pui[tsteps].data.swe.enpot,2,/nan) ;swea s/c potential corrected energy table (eV)
  yswepot=average(pui[tsteps].data.swe.efpot,2,/nan) ;swea pot eflux
  xswe=pui1.sweet ;swea energy table (eV)
  yswe=average(pui[tsteps].data.swe.eflux,2,/nan) ;swea eflux
  xswi=average(info_str[pui[tsteps].data.swi.swis.info_index].energy_coarse,2,/nan) ;swia energy table (eV)
  yswi=average(pui[tsteps].data.swi.swis.data,2,/nan) ;swia energy flux

  yswepot[where(yswepot eq 0,/null)]=zeros
  yswe[where(yswe eq 0,/null)]=zeros
  yswi[where(yswi eq 0,/null)]=zeros

  xr=[1,30e3]
  yr=[5e2,5e9]
  xt='Energy (eV)'
  yt='Eflux (eV/[cm2 s sr eV])'
  p=plot([0],/nodata,/xlog,/ylog,xrange=xr,yrange=yr,title='SWEA/SWIA',xtitle=xt,ytitle=yt,xtickunits='scientific',ytickunits='scientific')
  p=plot(/o,xswepot,yswepot,/stairs,'m')
  p=plot(/o,xswe,yswe,/stairs,'b')
  p=plot(/o,xswi,yswi,/stairs,'r')
endif

if keyword_set(traj) then begin ;Trajectories and Ring Distribution
  stxfov=average(pui[tsteps].data.sta.fov.x,2,/nan) ;static x-fov
  stzfov=average(pui[tsteps].data.sta.fov.z,2,/nan) ;static z-fov
  scp=1e-3*average(pui[tsteps].data.scp,2,/nan) ;spacecraft position (km)
  mag=average(pui[tsteps].data.mag.mso,2,/nan) ;mag field (T)
  magdir=mag/sqrt(total(mag^2)) ;mag direction
  usw=average(pui[tsteps].data.swi.swim.velocity_mso,2,/nan) ;solar wind velocity (km/s)
  uswtot=sqrt(total(usw^2)) ;solar wind speed (km/s)
  esw=crossp(-usw,magdir) ;motional electric field direction (km/s)
  drf=crossp(esw,magdir) ;drift velocity (km/s)
  alfrm=[0.,10.*3400.] ;fov arrow length factor in R_M (km)
  vmax=[0.,-max(sqrt(total(rv[*,1,3:5]^2,3)))] ;max pickup speed (km/s)

  npf=floor(pui0.np/1.1) ;fraction of number of simulated particles
  p=plot3d(rv[*,0,0],rv[*,0,1],rv[*,0,2],'b',axis_style=2) ;H birth curve
  p=plot3d(rv[0:npf,1,0],rv[0:npf,1,1],rv[0:npf,1,2],'r',/o) ;O birth curve
  p=plot3d(scp[0]+alfrm*sep1fov[0],scp[1]+alfrm*sep1fov[1],scp[2]+alfrm*sep1fov[2],/o,'b') ;SEP1F fov (blue)
  p=plot3d(scp[0]+alfrm*sep2fov[0],scp[1]+alfrm*sep2fov[1],scp[2]+alfrm*sep2fov[2],/o,'r') ;SEP2F fov (red)
  p=plot3d(scp[0]+alfrm*stxfov[0],scp[1]+alfrm*stxfov[1],scp[2]+alfrm*stxfov[2],/o,'c') ;static x fov (cyan)
  p=plot3d(scp[0]+alfrm*stzfov[0],scp[1]+alfrm*stzfov[1],scp[2]+alfrm*stzfov[2],/o,'m') ;static z fov (magenta)
  p=plot3d(scp[0]+alfrm*magdir[0],scp[1]+alfrm*magdir[1],scp[2]+alfrm*magdir[2],/o,'g') ;mag direction
  p=plot3d(scp[0]+alfrm*usw[0]/uswtot,scp[1]+alfrm*usw[1]/uswtot,scp[2]+alfrm*usw[2]/uswtot,/o,'k') ;solar wind velocity direction
  p=plot3d(scp[0]+alfrm*esw[0]/uswtot,scp[1]+alfrm*esw[1]/uswtot,scp[2]+alfrm*esw[2]/uswtot,/o,color=!color.orange) ;motional electric field direction
  p=plot3d(scp[0]+alfrm*drf[0]/uswtot,scp[1]+alfrm*drf[1]/uswtot,scp[2]+alfrm*drf[2]/uswtot,/o,color=!color.gray) ;drift velocity direction
  p=scatterplot3d([0,seprv[*,0]],[0,seprv[*,1]],[0,seprv[*,2]],/o,rgb=34,magnitude=[1,0,2]) ;SEP on birth curve (SEP1F: blue, SEP2F, red)
  mvn_pui_plot_mars_bow_shock,/p3d

  p=plot3d(rv[0:npf,1,3],rv[0:npf,1,4],rv[0:npf,1,5],axis_style=2,aspect_ratio=1,aspect_z=1,xtitle='X (km/s)',ytitle='Y (km/s)',ztitle='Z (km/s)') ;ring-beam distribution
  p=plot3d(vmax*sepfov[0,0],vmax*sepfov[1,0],vmax*sepfov[2,0],/o,'b') ;SEP1F reversed fov
  p=plot3d(vmax*sepfov[0,1],vmax*sepfov[1,1],vmax*sepfov[2,1],/o,'r') ;SEP2F reversed fov
  p=plot3d(vmax*stxfov[0],vmax*stxfov[1],vmax*stxfov[2],/o,'c') ;static x reversed fov
  p=plot3d(vmax*stzfov[0],vmax*stzfov[1],vmax*stzfov[2],/o,'m') ;static z reversed fov
  p=plot3d(-vmax*magdir[0],-vmax*magdir[1],-vmax*magdir[2],/o,'g') ;mag direction
  p=plot3d([0,usw[0]],[0,usw[1]],[0,usw[2]],/o,'k') ;solar wind velocity (km/s)
  p=plot3d([0,esw[0]],[0,esw[1]],[0,esw[2]],/o,color=!color.orange) ;motional electric field direction
  p=plot3d([0,drf[0]],[0,drf[1]],[0,drf[2]],/o,color=!color.gray) ;drift velocity (km/s)
  p=scatterplot3d([0,seprv[*,3]],[0,seprv[*,4]],[0,seprv[*,5]],/o,rgb=34,magnitude=[1,0,2]) ;SEP on ring (SEP1F: blue, SEP2F, red)
endif

if keyword_set(eflux) then begin ;Efluxes
  eftot=average(pui[tsteps].model.fluxes.toteflux,3,/nan) ;total flux /[s cm2 keV]
  eftot[where(eftot eq 0.,/null)]=zeros
  p=plot([0],/nodata,/xlog,/ylog,yrange=[1e1,1e6],xtitle=xt,ytitle='Eflux (eV/[cm2 s eV])',xtickunits='scientific',ytickunits='scientific')
  p=plot(pui1.totet,eftot[*,0],/o,color='b') ;pickup H+
  p=plot(pui1.totet,eftot[*,1],/o,color='r') ;pickup O+
  if pui0.ns ge 3 then p=plot(pui1.totet,eftot[*,2],/o,color='g') ;pickup (H2)+
endif

mvn_pui_tplot_3d,swia3d=swia3d,stah3d=stah3d,stao3d=stao3d,lineplot=tsteps
;stop
end